########################################################
### MCMC Algorithm to estimate parameters in model  ###
####    for Booms et al 2021 J. Raptor Research   #####
################ by: Joe Eisaguirre  ###################
############ Last edited: 10 January 2021 ##############
#########################################################


logit.reg.mcmc=function(y,z,X,W,beta.mn,beta.var,beta0.mn,beta0.var0,beta.tune,
                        alpha.mn,alpha.var,alpha0.mn,alpha0.var0,alpha.tune,
                        U,a.g,b.g,lambda.tune,lam.var0,lam.mn,
                        n.mcmc){
  
  ###
  ### Libraries and Subroutines
  ###
  
  library(invgamma)
  
  logit=function(p){
    log(p/(1-p))
  }
  
  logit.inv=function(z){
    exp(z)/(1+exp(z))
  }
  
  ###
  ### Preliminary variables
  ###
  
  n.burn=round(n.mcmc/10)
  n=length(y)
  nn=length(z)
  p=dim(X)[2]
  p1=dim(W)[2]
  
  beta.save=matrix(0,p,n.mcmc)
  beta0.save=matrix(0,1,n.mcmc)
  beta0.var.save=matrix(0,1,n.mcmc)
  
  alpha.save=matrix(0,p,n.mcmc)
  alpha0.save=matrix(0,1,n.mcmc)
  alpha.int.save=matrix(0,1,n.mcmc)
  beta.int.save=matrix(0,1,n.mcmc)
  alpha0.var.save=matrix(0,1,n.mcmc)
  Davg=0
  N.save=matrix(0,length(U),n.mcmc)
  N.f.save=matrix(0,1,n.mcmc)
  N.1y.save=matrix(0,1,n.mcmc)
  N.2y.save=matrix(0,1,n.mcmc)
  N.3y.save=matrix(0,1,n.mcmc)
  N.tot.save=matrix(0,1,n.mcmc)
  N.tot.state.l.save=matrix(0,1,n.mcmc)
  N.tot.state.ml.save=matrix(0,1,n.mcmc)
  lambda.save=matrix(0,length(U),n.mcmc)
  mu.lambda.save=matrix(0,1,n.mcmc)
  lam.var.save=matrix(0,1,n.mcmc)
  N.p.save=matrix(0,1,n.mcmc)
  
    
  ###
  ### Starting values
  ###
  
  Sig.beta.inv=solve(beta.var*diag(p))
  beta=beta.mn+2
  beta.int=beta0.mn
  beta0=beta0.mn
  beta0.var=beta0.var0
  pp=logit.inv(beta.int+X%*%beta)
  Sig.alpha.inv=solve(alpha.var*diag(p1))
  alpha=alpha.mn+3
  alpha.int=alpha0.mn
  alpha0=alpha0.mn
  alpha0.var=alpha0.var0
  ppa=logit.inv(alpha.int+W%*%alpha)
  
  N=U+100
  lambda=N
  mu.lambda=1300
  lam.var=1
  
  ff1=200
  ff2=200
  ff3=200
  
  ###
  ### Priors
  ###
  
  q=1
  r=1
  
  ###
  ### MCMC Loop
  ###
  
  for(k in 1:n.mcmc){
    
    ##
    ## Sample beta -- MH updated
    ##
    
    beta.star=rnorm(p,beta,beta.tune)
    pp.star=logit.inv(X%*%beta.star)
    p.ad.star=logit.inv(beta.star)*logit.inv(alpha)
    p.ad=logit.inv(beta)*logit.inv(alpha)
    mh1=sum(dbinom(y,1,pp.star,log=TRUE))+
      sum(dbinom(U,N,p.ad.star,log=TRUE))+
      sum(dnorm(beta.star,beta0,beta0.var^.5,log=TRUE))
    mh2=sum(dbinom(y,1,pp,log=TRUE))+
      sum(dbinom(U,N,p.ad,log=TRUE))+
      sum(dnorm(beta,beta0,beta0.var^.5,log=TRUE))
    mh=exp(mh1-mh2)
    if(mh>-Inf & mh>runif(1)){
      beta=beta.star
      pp=pp.star
    }
    

    ##
    ## Sample beta0 -- Gibbs update
    ##

    mu.var=1/(p/beta0.var+1/beta0.var0)
    mu.mean=mu.var*(sum(beta)/beta0.var+beta0.mn/beta0.var0)
    beta0=rnorm(1,mu.mean,mu.var^.5)
    
    ##
    ## Sample beta0.var -- Gibbs update
    ##
    
    tmp.r=1/(sum((beta-beta0)^2)/2+1/r)
    tmp.q=p/2+q
    beta0.var=1/rgamma(1,tmp.q,,tmp.r)
    
    
    ##
    ## Sample alpha -- MH update
    ##
    
    alpha.star=rnorm(p1,alpha,alpha.tune)
    ppa.star=logit.inv(W%*%alpha.star)
    p.ad.star=logit.inv(beta)*logit.inv(alpha.star)
    p.ad=logit.inv(beta)*logit.inv(alpha)
    mh1=sum(dbinom(z,1,ppa.star,log=TRUE))+
      sum(dbinom(U,N,p.ad.star,log=TRUE))+
      sum(dnorm(alpha.star,alpha0,alpha0.var^.5,log=TRUE))
    mh2=sum(dbinom(z,1,ppa,log=TRUE))+
      sum(dbinom(U,N,p.ad,log=TRUE))+
      sum(dnorm(alpha,alpha0,alpha0.var^.5,log=TRUE))
    mh=exp(mh1-mh2)
    if(mh>-Inf & mh>runif(1)){
      alpha=alpha.star
      ppa=ppa.star
    }
    
    ##
    ## Sample alpha0 -- Giibs
    ##
    
    mu.var=1/(p1/alpha0.var+1/alpha0.var0)
    mu.mean=mu.var*(sum(alpha)/alpha0.var+alpha0.mn/alpha0.var0)
    alpha0=rnorm(1,mu.mean,mu.var^.5)
    
    ##
    ## Sample alpha0.var -- Gibbs
    ##
    
    tmp.r=1/(sum((alpha-alpha0)^2)/2+1/r)
    tmp.q=p1/2+q
    alpha0.var=1/rgamma(1,tmp.q,,tmp.r)
    
    ###
    ### sample lambda -- MH
    ###
    
    #lambda=rgamma(length(N),N+a.g,1+b.g)
    lambda.star=rnorm(length(U),lambda,lambda.tune)
    if(all(lambda.star>0)){
    mh1=sum(dpois(N,lambda.star,log=TRUE))+
      sum(dnorm(lambda.star,mu.lambda,lam.var,log=TRUE))
    mh2=sum(dpois(N,lambda,log=TRUE))+
      sum(dnorm(lambda,mu.lambda,lam.var,log=TRUE))
    mh=exp(mh1-mh2)
    if(mh>runif(1)){
      lambda=lambda.star
    }
    }
    
    ###
    ### sample mu.lambda -- Gibbs
    ###
    
    mu.var=1/(p1/lam.var+1/lam.var0)
    mu.mean=mu.var*(sum(lambda)/lam.var+lam.mn/lam.var0)
    mu.lambda=rnorm(1,mu.mean,mu.var^.5)
    
    ##
    ## Sample lam.var -- Gibbs
    ##
    
    tmp.r=1/(sum((lambda-mu.lambda)^2)/2+1/r)
    tmp.q=p1/2+q
    lam.var=1/rgamma(1,tmp.q,,tmp.r)
    
    ###
    ### N.pool global abundance from posterior predictive
    ###
    
    N.p=rpois(1,mu.lambda)
    
    
    ###
    ### sample annual abundance -- MH
    ###
    
    N.star=N+sample(-20:20,length(N))
    p.ad=logit.inv(beta)*logit.inv(alpha)
    mh1=sum(dbinom(U,N.star,p.ad,log=TRUE))+
      sum(dpois(N.star,lambda,log=TRUE))
    mh2=sum(dbinom(U,N,p.ad,log=TRUE))+
      sum(dpois(N,lambda,log=TRUE))
    mh=exp(mh1-mh2)
    if(mh>runif(1)){
      N=N.star
    }
    
    ###
    ### sample annual fledglings
    ###
    
    prod=rnorm(1,.55,.087) # .55 productivty estimate
    
    pt=rnorm(1,1/2.13,.01)  # prop adults occ territory, 1.13:1 floater:breeder from USFWS 2016
    
    pairs1=N.p*pt/2*(1180489/108945)  # territorial stateide
    pairs2=N.p*pt/2*(1180489/150325)
    
    if(prod>=0&pt>=0){
      ff1=rpois(1,prod*pairs1)  
      ff2=rpois(1,prod*pairs2)  
    }
    
    N.f1=c(ff1)
    N.f2=c(ff2)
    
    ###
    ### sample age-specific survival rates
    ###
    
    s1=rbeta(1,353,159)  ## from USFWS 2016
    s2=rbeta(1,287,85)
    s3=rbeta(1,243,47)
    
    ###
    ### age-specific abundances 
    ###
    
    N.ad.s1=rpois(1,N.p*(1180489/108945))
    N.1y1=rpois(1,s1*N.f1)
    N.2y1=rpois(1,s2*N.1y1)
    N.3y1=rpois(1,s3*N.2y1)
    
    N.ad.s2=rpois(1,N.p*(1180489/150325))
    N.1y2=rpois(1,s1*N.f2)
    N.2y2=rpois(1,s2*N.1y2)
    N.3y2=rpois(1,s3*N.2y2)
    
    ###
    ### total abundance
    ###
    
    N.tot=1
    N.tot.state.l=N.ad.s1+N.1y1+N.2y1+N.3y1+N.f1
    N.tot.state.ml=N.ad.s2+N.1y2+N.2y2+N.3y2+N.f2 
    
    ##
    ## DIC calcs (not used)
    ##
    
    if(k>n.burn){
      Davg=Davg-2*(sum(dbinom(y,1,pp,log=TRUE)))/
        (n.mcmc-n.burn)
    }
    
    ##
    ## Save samples
    ##
    
    beta.save[,k]=beta
    beta0.save[,k]=beta0
    beta0.var.save[,k]=beta0.var
    alpha.int.save[,k]=alpha.int
    beta.int.save[,k]=beta.int
    
    alpha.save[,k]=alpha
    alpha0.save[,k]=alpha0
    alpha0.var.save[,k]=alpha0.var
    N.save[,k]=N
    N.f.save[,k]=N.f1
    N.1y.save[,k]=N.1y1
    N.2y.save[,k]=N.2y1
    N.3y.save[,k]=N.3y1
    N.tot.save[,k]=N.tot
    N.tot.state.l.save[,k]=N.tot.state.l
    N.tot.state.ml.save[,k]=N.tot.state.ml
    lambda.save[,k]=lambda
    mu.lambda.save[,k]=mu.lambda
    lam.var.save[,k]=lam.var
    N.p.save[,k]=N.p
    
  }
  
  ###
  ### Calc DIC (not used)
  ###
  
  postbeta.mn=apply(beta.save[,-(1:n.burn)],1,mean)
  Dhat=-2*(sum(dbinom(y,1,logit.inv(X%*%postbeta.mn),
                      log=TRUE)))
  pD=Davg-Dhat
  DIC=2*Davg-Dhat
  
  ###
  ### Write output
  ###
  
  list(y=y,X=X,n.mcmc=n.mcmc,beta.save=beta.save,alpha.save=alpha.save,alpha.int.save=alpha.int.save,beta.int.save=beta.int.save,
       beta0.save=beta0.save,beta0.var.save=beta0.var.save,
       alpha0.save=alpha0.save,alpha0.var.save=alpha0.var.save,N.save=N.save,lambda.save=lambda.save,
       N.f.save=N.f.save,N.1y.save=N.1y.save,N.2y.save=N.2y.save,N.3y.save=N.3y.save,N.tot.save=N.tot.save,
       N.tot.state.l.save=N.tot.state.l.save,N.tot.state.ml.save=N.tot.state.ml.save,mu.lambda.save=mu.lambda.save,
       lam.var.save=lam.var.save,N.p.save=N.p.save,
       DIC=DIC)
  
}
